@@ -1,12 +1,13 @@
-package App::Pinto;
-
 # ABSTRACT: Command-line driver for Pinto
 
+package App::Pinto;
+
 use strict;
 use warnings;
 
+use Encode;
 use Class::Load;
-
+use Term::Prompt;
 use List::Util qw(min);
 use Log::Dispatch::Screen;
 use Log::Dispatch::Screen::Color;
@@ -17,17 +18,19 @@ use App::Cmd::Setup -app;
 
 #------------------------------------------------------------------------------
 
-our $VERSION = '0.042'; # VERSION
+our $VERSION = '0.043'; # VERSION
 
 #------------------------------------------------------------------------------
 
 sub global_opt_spec {
 
     return (
-        [ 'root|r=s'    => 'Path to your repository root directory'  ],
-        [ 'nocolor'     => 'Do not colorize diagnostic messages'     ],
-        [ 'quiet|q'     => 'Only report fatal errors'                ],
-        [ 'verbose|v+'  => 'More diagnostic output (repeatable)'     ],
+        [ 'root|r=s'     => 'Path to your repository root directory'  ],
+        [ 'nocolor'      => 'Do not colorize diagnostic messages'     ],
+        [ 'password|p=s' => 'Password for server authentication'      ],
+        [ 'quiet|q'      => 'Only report fatal errors'                ],
+        [ 'username|u=s' => 'Username for server authentication'      ],
+        [ 'verbose|v+'   => 'More diagnostic output (repeatable)'     ],
     );
 
     # TODO: Add options for color contols!
@@ -40,19 +43,26 @@ sub pinto {
     my ($self) = @_;
 
     return $self->{pinto} ||= do {
-        my %global_options = %{ $self->global_options() };
+        my $global_options = $self->global_options;
 
-        $global_options{root} ||= $ENV{PINTO_REPOSITORY_ROOT}
+        $global_options->{root} ||= $ENV{PINTO_REPOSITORY_ROOT}
             || $self->usage_error('Must specify a repository root');
 
+        $global_options->{password} = $self->_prompt_for_password
+            if defined $global_options->{password} and $global_options->{password} eq '-';
+
+        # Translate (progressive) verbose value into a (regressive) log_level value
+        $global_options->{log_level} = 3 - min(delete $global_options->{verbose} || 0, 3);
+        $global_options->{log_level} = 4 if delete $global_options->{quiet};
+
         # TODO: Give helpful error message if the right backend
         # is not installed.
 
-        my $pinto_class = $self->pinto_class($global_options{root});
+        my $pinto_class = $self->pinto_class_for($global_options->{root});
         Class::Load::load_class($pinto_class);
 
-        my $pinto = $pinto_class->new(%global_options);
-        $pinto->add_logger($self->logger(%global_options));
+        my $pinto = $pinto_class->new( %{ $global_options } );
+        $pinto->add_logger($self->make_logger( %{ $global_options } ));
 
         $pinto;
     };
@@ -60,25 +70,22 @@ sub pinto {
 
 #------------------------------------------------------------------------------
 
-sub pinto_class {
+sub pinto_class_for {
     my ($self, $root) = @_;
     return $root =~ m{^http://}x ? 'Pinto::Remote' : 'Pinto';
 }
 
 #------------------------------------------------------------------------------
 
-sub logger {
+sub make_logger {
     my ($self, %options) = @_;
 
     my $nocolor   = $options{nocolor};
     my $colors    = $nocolor ? {} : ($self->log_colors);
     my $log_class = 'Log::Dispatch::Screen';
-    $log_class .= '::Color' unless $nocolor;
+    $log_class   .= '::Color' unless $nocolor;
 
-    my $verbose = min($options{verbose} || 0, 2);
-
-    my $log_level = 2 - $verbose;      # Defaults to 'notice'
-    $log_level = 4 if $options{quiet}; # Only 'error' or higher
+    my $log_level = $options{log_level};
 
     return $log_class->new( min_level => $log_level,
                             color     => $colors,
@@ -103,6 +110,19 @@ sub default_log_colors { return $PINTO_DEFAULT_LOG_COLORS }
 
 #------------------------------------------------------------------------------
 
+sub _prompt_for_password {
+   my ($self) = @_;
+
+   my $input    = Term::Prompt::prompt('p', 'Password:', '', '');
+   my $password = Encode::decode_utf8($input);
+   print "\n"; # Get on a new line
+
+   return $password;
+}
+
+#-------------------------------------------------------------------------------
+
+
 1;
 
 
@@ -117,7 +137,7 @@ App::Pinto - Command-line driver for Pinto
 
 =head1 VERSION
 
-version 0.042
+version 0.043
 
 =head1 DESCRIPTION
 
